@@ -13,8 +13,7 @@ from traitlets import Unicode, Dict, List, Tuple, Integer, Float, Any, Bool, obs
 from IPython.display import HTML, update_display
 from pyparsing import ParseException
 
-from .utils import get_parser, to_json
-from .boundingbox import combined_bb, normalize
+from .utils import get_parser, to_json, bsphere, normalize
 
 
 class AnimationTrack:
@@ -111,8 +110,8 @@ class CadViewerWidget(widgets.Output):  # pylint: disable-msg=too-many-instance-
     _model_name = Unicode("CadViewerModel").tag(sync=True)
     _view_module = Unicode("cad-viewer-widget").tag(sync=True)
     _model_module = Unicode("cad-viewer-widget").tag(sync=True)
-    _view_module_version = Unicode("1.1.1").tag(sync=True)
-    _model_module_version = Unicode("1.1.1").tag(sync=True)
+    _view_module_version = Unicode("1.3.5").tag(sync=True)
+    _model_module_version = Unicode("1.3.5").tag(sync=True)
 
     #
     # Display traits
@@ -158,12 +157,18 @@ class CadViewerWidget(widgets.Output):  # pylint: disable-msg=too-many-instance-
     tools = Bool(allow_none=True, default_value=None).tag(sync=True)
     "bool: Whether to show CAD tools (True) or not (False)"
 
+    glass = Bool(allow_none=True, default_value=None).tag(sync=True)
+    "bool: Whether to use the glass mode (CAD navigation as transparent overlay) or not"
+
     ortho = Bool(allow_none=True, default_value=None).tag(sync=True)
     "bool: Whether to use orthographic view (True) or perspective view (False)"
 
     control = Unicode().tag(sync=True)
     "unicode: Whether to use trackball controls ('trackball') or orbit controls ('orbit')"
 
+    up = Unicode().tag(sync=True)
+    "unicode: Whether camera up direction is Z ('Z') or Y ('Y')"
+
     axes = Bool(allow_none=True, default_value=None).tag(sync=True)
     "bool: Whether to show coordinate axes (True) or not (False)"
 
@@ -182,6 +187,9 @@ class CadViewerWidget(widgets.Output):  # pylint: disable-msg=too-many-instance-
     black_edges = Bool(allow_none=True, default_value=None).tag(sync=True)
     "bool: Whether to shows the edges in black (True) or not(False)"
 
+    collapse = Integer(allow_none=True, default_value=None).tag(sync=True)
+    "int: Collapse CAD tree (1: collapse nodes with single leaf, 2: collapse all nodes)"
+
     normal_len = Float(allow_none=True).tag(sync=True)
     "float: If > 0, the vertex normals will be rendered with the length given be this parameter"
 
@@ -330,6 +338,8 @@ class CadViewer:
         UI theme, can be 'dark' or 'light' (default)
     tools : bool, default: True
         Whether to show CAD tools (True) or not (False)
+    glass : bool, default: False
+        Whether to use glass mode (True) or not (False)
     pinning: bool, default: False
         Whether to allow replacing the CAD View by a canvas screenshot
 
@@ -346,13 +356,15 @@ class CadViewer:
         cad_width=800,
         height=600,
         tree_width=240,
-        theme="light",
+        theme="browser",
+        glass=False,
+        tools=True,
         pinning=False,
         title=None,
         anchor=None,
     ):
-        if cad_width < 640:
-            raise ValueError("Ensure cad_width >= 640")
+        if cad_width < 400:
+            raise ValueError("Ensure cad_width >= 400")
         if tree_width < 240:
             raise ValueError("Ensure tree_width >= 240")
 
@@ -361,6 +373,8 @@ class CadViewer:
             height=height,
             tree_width=tree_width,
             theme=theme,
+            glass=glass,
+            tools=tools,
             pinning=pinning,
             title=title,
             anchor=anchor,
@@ -398,8 +412,13 @@ class CadViewer:
         ambient_intensity=0.5,
         direct_intensity=0.3,
         # viewer options
-        tools=True,
+        tools=None,
+        glass=None,
+        cad_width=None,
+        tree_width=None,
+        height=None,
         control="trackball",
+        up="Z",
         ortho=True,
         axes=False,
         axes0=False,
@@ -407,10 +426,11 @@ class CadViewer:
         ticks=10,
         transparent=False,
         black_edges=False,
+        collapse=0,
         position=None,
         quaternion=None,
         target=None,
-        zoom=1.0,
+        zoom=None,
         reset_camera=True,
         zoom_speed=1.0,
         pan_speed=1.0,
@@ -433,8 +453,20 @@ class CadViewer:
             Name of the title view to display the shapes.
         ortho : bool, default True
             Whether to use orthographic view (True) or perspective view (False)
+        cad_width : int, default: None
+            Width of the canvas element
+        height : int, default: None
+            Height of the canvas element
+        tree_width : int, default: None
+            Width of the navigation tree element
+        tools : bool, default: None
+            Whether to show CAD tools (True) or not (False)
+        glass : bool, default: None
+            Whether to use glass mode (True) or not (False)
         control : string, default 'trackball'
             Whether to use trackball controls ('trackball') or orbit controls ('orbit')
+        up : string, default 'Z'
+            Whether camera up direction is Z ('Z') or Y ('Y')
         axes : bool, default False
             Whether to show coordinate axes (True) or not (False)
         axes0 : bool, default False
@@ -447,6 +479,8 @@ class CadViewer:
             Whether to show the CAD objects transparently (True) or not (False)
         black_edges : bool, default False
             Whether to shows the edges in black (True) or not(False)
+        collapse : int, default 0
+            Collapse CAD tree (1: collapse nodes with single leaf, 2: collapse all nodes)
         normal_Len : int, default 0
             If > 0, the vertex normals will be rendered with the length given be this parameter
         default_edge_color : string, default "#707070"
@@ -486,7 +520,12 @@ class CadViewer:
         shapes = {
             "name": "Group",
             "id": "/Group",
-            "loc": None,  # would be (<position>, <quaternion>), e.g. ([0,0,0), (0,0,0,1)])
+            "loc": None,  # would be (<position>, <quaternion>), e.g. ([0,0,0), (0,0,0,1)]),
+            "bb": {
+                "xmin": -0.5, "xmax": 0.5,
+                "ymin": -0.5, "ymax": 0.5,
+                "zmin": -0.5, "zmax": 0.5
+            }
             "parts": [{
                 "name": "Part_0",
                 "id": "/Group/Part_0",
@@ -524,12 +563,7 @@ class CadViewer:
                     [[-0.5, 0.5, 0.5], [0.5, 0.5, 0.5]]
                 ]},
                 "color": "#e8b024",
-                "renderback": false,
-                "bb": {
-                    "xmin": -0.5, "xmax": 0.5,
-                    "ymin": -0.5, "ymax": 0.5,
-                    "zmin": -0.5, "zmax": 0.5
-                }
+                "renderback": false
             }]
         }
         states = {'/Group/Part_0': [1, 1]}
@@ -602,12 +636,7 @@ class CadViewer:
             "name": "<str>",
             "type": "shapes",
             "color": "#ffffff",
-            "renderback": false,
-            "bb": {
-                "xmin": -0.5, "xmax": 0.5,
-                "ymin": -0.5, "ymax": 0.5,
-                "zmin": -0.5, "zmax": 0.5
-            },
+            "renderback": false
             "shape": {
                 "vertices": <VectorList>,
                 "triangles": <Index>,
@@ -622,12 +651,7 @@ class CadViewer:
             "type": "edges",
             "color": "#ffffff",
             "width": 3,
-            "renderback": false,
-            "bb": {
-                "xmin": -0.5, "xmax": 0.5,
-                "ymin": -0.5, "ymax": 0.5,
-                "zmin": -0.5, "zmax": 0.5
-            },
+            "renderback": false
             "shape": <EdgeList>
         }
 
@@ -636,12 +660,7 @@ class CadViewer:
             "name": "<name>",
             "type": "vertices",
             "color": "#ffffff",
-            "size": 6,
-            "bb": {
-                "xmin":  -5.0, "xmax": 5.0,
-                "ymin": -10.0, "ymax": 10.0,
-                "zmin": -15.0, "zmax": 15.0
-            },
+            "size": 6
             "shape": <VectorList>
         }
         """
@@ -652,6 +671,9 @@ class CadViewer:
         if control == "trackball" and position is not None and quaternion is None:
             raise ValueError("For Trackball camera control, position paramater also needs quaternion parameter")
 
+        if up != "Z" and up != "Y":
+            raise ValueError("Camera up value can only be Y or Z")
+
         if grid is None:
             grid = [False, False, False]
 
@@ -665,26 +687,27 @@ class CadViewer:
                 print("Camera control changed, so camera was resetted")
 
         if reset_camera:
-            bb = combined_bb(shapes)
+            center, radius = bsphere(shapes["bb"])
 
             if position is None:
-                position = (
-                    normalize(np.array((1, 1, 1))) * 5.5 * bb.max_dist_from_center() + np.array(bb.center)
-                ).tolist()
+                position = (normalize(np.array((1, 1, 1))) * 5.5 * radius + center).tolist()
 
             if quaternion is None and control == "trackball":
-                quaternion = (
-                    0.1759198966061612,
-                    0.42470820027786693,
-                    0.8204732385702833,
-                    0.33985114297998736,
-                )
+                if up == "Y":
+                    quaternion = (-0.27984814233312133, 0.3647051996310009, 0.11591689595929514, 0.8804762392171493)
+                else:  # Z as default
+                    quaternion = (0.1759198966061612, 0.42470820027786693, 0.8204732385702833, 0.33985114297998736)
 
             if target is None:
-                target = bb.center
+                target = center.tolist()
 
             if zoom is None:
-                zoom = 1.0
+                zoom = 4 / 3
+                w = self.widget.cad_width if cad_width is None else cad_width
+                h = self.widget.height if height is None else height
+                if w >= h:
+                    zoom *= h / w
+
         else:
             copy = lambda v: None if v is None else (*v,)
             position = copy(self.widget.position)
@@ -703,9 +726,18 @@ class CadViewer:
             self.widget.ambient_intensity = ambient_intensity
             self.widget.direct_intensity = direct_intensity
             self.widget.normal_len = normal_len
-
-            self.widget.tools = tools
             self.widget.control = control
+            self.widget.up = up
+            if tools is not None:
+                self.widget.tools = tools
+            if glass is not None:
+                self.widget.glass = glass
+            if cad_width is not None:
+                self.widget.cad_width = cad_width
+            if tree_width is not None:
+                self.widget.tree_width = tree_width
+            if height is not None:
+                self.widget.height = height
             self.widget.axes = axes
             self.widget.axes0 = axes0
             self.widget.grid = grid
@@ -713,6 +745,7 @@ class CadViewer:
             self.widget.ortho = ortho
             self.widget.transparent = transparent
             self.widget.black_edges = black_edges
+            self.widget.collapse = collapse
             self.widget.reset_camera = reset_camera
             self.widget.position = position
             self.widget.quaternion = quaternion
@@ -733,11 +766,19 @@ class CadViewer:
             self.add_tracks(tracks)
 
         self.widget.initialize = False
-        # self.update_camera_location()
+
+        if tools is not None:
+            self.widget.tools = tools
+
+        if glass is not None:
+            self.widget.glass = glass
 
     def update_states(self, states):
         """Set navigation tree states for a CAD view"""
-
+        all_paths = list(self.widget.states.keys())
+        for path, state in states.items():
+            if not path in all_paths:
+                raise ValueError(f"Path {path} is not a valid state")
         self.widget.state_updates = states
 
     def update_camera_location(self):
@@ -1036,6 +1077,58 @@ class CadViewer:
     def tools(self, value):
         self.widget.tools = value
 
+    @property
+    def glass(self):
+        """
+        Get or set the CadViewerWidget traitlet `glass`
+        see [CadViewerWidget.tools](./widget.html#cad_viewer_widget.widget.CadViewerWidget.glass)
+        """
+
+        return self.widget.glass
+
+    @glass.setter
+    def glass(self, value):
+        self.widget.glass = value
+
+    @property
+    def cad_width(self):
+        """
+        Get or set the CadViewerWidget traitlet `cad_width`
+        see [CadViewerWidget.tools](./widget.html#cad_viewer_widget.widget.CadViewerWidget.cad_width)
+        """
+
+        return self.widget.cad_width
+
+    @cad_width.setter
+    def cad_width(self, value):
+        self.widget.cad_width = value
+
+    @property
+    def tree_width(self):
+        """
+        Get or set the CadViewerWidget traitlet `tree_width`
+        see [CadViewerWidget.tools](./widget.html#cad_viewer_widget.widget.CadViewerWidget.tree_width)
+        """
+
+        return self.widget.tree_width
+
+    @tree_width.setter
+    def tree_width(self, value):
+        self.widget.tree_width = value
+
+    @property
+    def height(self):
+        """
+        Get or set the CadViewerWidget traitlet `height`
+        see [CadViewerWidget.tools](./widget.html#cad_viewer_widget.widget.CadViewerWidget.height)
+        """
+
+        return self.widget.height
+
+    @height.setter
+    def height(self, value):
+        self.widget.height = value
+
     @property
     def pan_speed(self):
         """
@@ -1157,6 +1250,15 @@ class CadViewer:
 
         return self.widget.control
 
+    @property
+    def up(self):
+        """
+        Get or set the CadViewerWidget traitlet `up`
+        see [CadViewerWidget.up](./widget.html#cad_viewer_widget.widget.CadViewerWidget.up)
+        """
+
+        return self.widget.up
+
     @property
     def pinning(self):
         """
@@ -1289,6 +1391,7 @@ class CadViewer:
         """
 
         self.execute("saveAsPng", filename)
+        self.execute("viewer.update", True)
 
     #
     # Tab handling
@@ -1447,6 +1550,9 @@ class CadViewer:
             args = [args]
         return wrapper()
 
+    def remove_ui_elements(self, elements):
+        self.execute("viewer.trimUI", [elements, False])
+
     def dump_model(self, shapes=False):
         print(
             dedent(
@@ -1475,14 +1581,17 @@ class CadViewer:
                             VIEWER
                 timeit:             {self.widget.timeit}
                 tools:              {self.widget.tools}
+                glass:              {self.widget.glass}
                 ortho:              {self.widget.ortho}
                 control:            {self.widget.control}
+                up:                 {self.widget.up}
                 axes:               {self.widget.axes}
                 axes0:              {self.widget.axes0}
                 grid:               {self.widget.grid}
                 ticks:              {self.widget.ticks}
                 transparent:        {self.widget.transparent}
                 black_edges:        {self.widget.black_edges}
+                collapse:           {self.widget.collapse}
                 tab:                {self.widget.tab}
                 clip_intersection:  {self.widget.clip_intersection}
                 clip_planes:        {self.widget.clip_planes}
