@@ -25,9 +25,9 @@ export class CadViewerModel extends DOMWidgetModel {
 
       title: null,
       anchor: null,
-      cadWidth: null,
+      cad_width: null,
       height: null,
-      treeWidth: null,
+      tree_width: null,
       theme: null,
       pinning: null,
 
@@ -38,15 +38,18 @@ export class CadViewerModel extends DOMWidgetModel {
       tracks: null,
       timeit: null,
       tools: null,
+      glass: null,
 
       ortho: null,
       control: null,
+      up: null,
       axes: null,
       axes0: null,
       grid: null,
       ticks: null,
       transparent: null,
       black_edges: null,
+      collapse: null,
       normal_len: null,
 
       default_edge_color: null,
@@ -110,6 +113,7 @@ export class CadViewerView extends DOMWidgetView {
     this.lastTarget = null;
     this.lastZoom = null;
     this.empty = true;
+    this.activeTab = "";
   }
 
   debug(...args) {
@@ -133,7 +137,12 @@ export class CadViewerView extends DOMWidgetView {
       this.model.on("change:ortho", this.handle_change, this);
       this.model.on("change:transparent", this.handle_change, this);
       this.model.on("change:black_edges", this.handle_change, this);
+      this.model.on("change:collapse", this.handle_change, this);
       this.model.on("change:tools", this.handle_change, this);
+      this.model.on("change:glass", this.handle_change, this);
+      this.model.on("change:cad_width", this.handle_change, this);
+      this.model.on("change:tree_width", this.handle_change, this);
+      this.model.on("change:height", this.handle_change, this);
       this.model.on("change:pinning", this.handle_change, this);
       this.model.on("change:default_edge_color", this.handle_change, this);
       this.model.on("change:default_opacity", this.handle_change, this);
@@ -192,6 +201,8 @@ export class CadViewerView extends DOMWidgetView {
       height: this.model.get("height"),
       treeWidth: this.model.get("tree_width"),
       theme: this.model.get("theme"),
+      glass: this.model.get("glass"),
+      tools: this.model.get("tools"),
       pinning: this.model.get("pinning")
     };
   }
@@ -209,7 +220,9 @@ export class CadViewerView extends DOMWidgetView {
   getViewerOptions() {
     var options = {
       control: this.model.get("control"),
+      up: this.model.get("up"),
       tools: this.model.get("tools"),
+      glass: this.model.get("glass"),
       axes: this.model.get("axes"),
       axes0: this.model.get("axes0"),
       grid: this.model.get("grid").slice(), // clone the array to ensure changes get detected
@@ -217,6 +230,7 @@ export class CadViewerView extends DOMWidgetView {
       ticks: this.model.get("ticks"),
       transparent: this.model.get("transparent"),
       blackEdges: this.model.get("black_edges"),
+      collapse: this.model.get("collapse"),
       timeit: this.model.get("timeit"),
       zoomSpeed: this.model.get("zoom_speed"),
       panSpeed: this.model.get("pan_speed"),
@@ -278,8 +292,12 @@ export class CadViewerView extends DOMWidgetView {
       true
     );
 
-    this.viewer.display.setAnimationControl(false);
-    this.viewer.display.setTools(options.tools);
+    this.viewer.display.showAnimationControl(false);
+    this.viewer.display.showTools(options.tools);
+
+    if (options.glass != null && this.title != null) {
+      this.viewer.display.glassMode(options.glass);
+    }
   }
 
   handleNotification(change) {
@@ -397,8 +415,14 @@ export class CadViewerView extends DOMWidgetView {
     this.tracks = [];
 
     var viewerOptions = this.getViewerOptions();
-
     timer.split("viewer");
+    
+    // set the latest view dimension before rendering
+    this.viewer.cadWidth = this.model.get("cad_width");
+    this.viewer.treeWidth = this.model.get("tree_width");
+    this.viewer.height = this.model.get("height");
+    this.viewer.glass = this.model.get("glass");
+
     this.viewer.render(
       ...this.viewer.renderTessellatedShapes(
         this.shapes,
@@ -408,6 +432,14 @@ export class CadViewerView extends DOMWidgetView {
       this.states,
       viewerOptions
     );
+    
+    // and resize the view accordingly afterwards
+    this.viewer.resizeCadView(
+      this.model.get("cad_width"),
+      this.model.get("tree_width"),
+      this.model.get("height"),
+      this.model.get("glass")
+    );
 
     timer.split("renderer");
 
@@ -525,6 +557,7 @@ export class CadViewerView extends DOMWidgetView {
 
     var tracks = "";
     var value = null;
+    var flag = null;
 
     switch (key) {
       case "zoom":
@@ -557,12 +590,53 @@ export class CadViewerView extends DOMWidgetView {
       case "black_edges":
         setKey("getBlackEdges", "setBlackEdges", key);
         break;
+      case "collapse":
+        var val = change.changed[key];
+        if (val == 1) {
+          this.viewer.display.collapseNodes("1");
+        } else if (val == 2) {
+          this.viewer.display.collapseNodes("C");
+        } else {
+          this.viewer.display.collapseNodes("E");
+        }
+        break;
       case "tools":
-        setKey("getTools", "setTools", key);
+        setKey("getTools", "showTools", key);
+        break;
+      case "glass":
+        flag = change.changed[key];
+        this.viewer.display.glassMode(flag);
+        break;
+      case "cad_width":
+        value = change.changed[key];
+        this.viewer.resizeCadView(
+          value,
+          this.model.get("tree_width"),
+          this.model.get("height"),
+          this.model.get("glass")
+        );
+        break;
+      case "tree_width":
+        value = change.changed[key];
+        this.viewer.resizeCadView(
+          this.model.get("cad_width"),
+          value,
+          this.model.get("height"),
+          this.model.get("glass")
+        );
+        break;
+      case "height":
+        value = change.changed[key];
+        this.viewer.resizeCadView(
+          this.model.get("cad_width"),
+          this.model.get("tree_width"),
+          value,
+          this.model.get("glass")
+        );
         break;
       case "pinning":
-        var flag = change.changed[key];
-        this.viewer.display.setPinning(flag);
+        flag = change.changed[key];
+        this.viewer.display.showPinning(flag);
         break;
       case "default_edge_color":
         setKey("getEdgeColor", "setEdgeColor", key);
@@ -598,10 +672,13 @@ export class CadViewerView extends DOMWidgetView {
         break;
       case "tab":
         value = change.changed[key];
-        if (value === "tree" || value == "clip") {
-          this.viewer.display.selectTabByName(value);
-        } else {
-          console.error(`cad-viewer-widget: unkonwn tab name ${value}`);
+        if (this.activeTab !== value) {
+          this.activeTab = value;
+          if (value === "tree" || value == "clip") {
+            this.viewer.display.selectTabByName(value);
+          } else {
+            console.error(`cad-viewer-widget: unkonwn tab name ${value}`);
+          }
         }
         break;
       case "clip_intersection":
